# -*- coding: utf-8 -*-
# vim: expandtab
import torch
from transformers import GPT2Tokenizer
from transformers import GPT2LMHeadModel
import tokenizers
import random
import sys
import time
from tokenizations import tokenization_bert_word_level as tokenization_bert

import argparse

if torch.cuda.is_available():
    device="cuda"
else:
    device="cpu"
device="cpu"


def choose_token(token_list):
    for i in range(len(token_list)):
        if token_list[i].item() == 4:
            continue
        return token_list[i].item()

def token_to_text(tokenizer, id):
    return tokenizer.decode(id)

def select_top_k(predictions, k=10):
    global tokenizer
    last=len(predictions)-1
    if last < 0:
        print("Error, prediction is 0")
        sys.exit(0)

    sorted_p = torch.softmax(predictions[last].sort(descending=True)[0],0)
    sorted_i = predictions[last].sort(descending=True)[1][:k]
    return sorted_i, sorted_p

def gen_block(text, prob):
    msg="<tr>"
    msg+="<td><div style='background-color: green' onclick='javascript: addText(\""+text+"\");'>"
    msg+=text
    msg+="</div></td>"
    msg+="<td><div style='background-color: orange'>"
    msg+= format(prob, ".3f")
    msg+="</div></td>"
    msg+="</tr>"
    return msg

def query_handler(furl_path):
    f = furl_path
    out=""
    try:
        print(f.args['query'])
    except:
        return ""
    name=f.args['name']
    query=f.args['query']
    idx,prob = infer(query, k=10)
    out += "<table border=1>"

    opts={}
    for j in range(len(idx)):
        v = token_to_text(tokenizer, idx[j].item()).replace(" ", "")
        p = prob[j].item()
        out+=gen_block(v, p)
        opts[j] = [ v, format(p, ".3f") ]
    out+="</table>"
    json_object = json.dumps(opts, indent = 4)
    out+="<script> opts[\"" + name + "\"]=\n" + json_object + "\n"
    return out






def infer(msg, k=5):
    total_predicted_text = msg
    indexed_tokens = tokenizer.encode(total_predicted_text)
    #print(indexed_tokens)
    tokens_tensor = torch.tensor(indexed_tokens).to(device)

    with torch.no_grad():
        outputs = model(input_ids=tokens_tensor)
        predictions = outputs[0]


    return select_top_k(predictions, k)

parser = argparse.ArgumentParser()
parser.add_argument('--model', default='', type=str, required=True, help='')
parser.add_argument('--vocab', default='', type=str, required=True, help='')
parser.add_argument('--vocab_type', default="bert", type=str, required=True, help='')
args = parser.parse_args()

print(args.vocab)
if args.vocab_type == "gpt2":
    tokenizer = GPT2Tokenizer.from_pretrained(args.vocab)
else:
    tokenizer = tokenization_bert.BertTokenizer(vocab_file=args.vocab)
print("Loading gpt2 model..." + args.model, end="")
#model = GPT2LMHeadModel.from_pretrained('gpt2')
print(args.model)
model = GPT2LMHeadModel.from_pretrained(args.model).to(device)
model.eval()
print("Done")



text =  "A bartender is "
ts=time.time()

for i in range(10):
    idx,prob = infer(text, k=5)
    print("Round " + str(i) + ":")
    print("    Current: " + text)
    for j in range(len(idx)):
        print("#%20s# : %.2f" % (token_to_text(tokenizer, idx[j].item()).replace(" ", ""), prob[j].item()))
    text += token_to_text(tokenizer, choose_token(idx))
print(text)
te=time.time()
print("%.5f sec" % (te-ts))

